@@ -90,13 +90,12 @@ cl_stm8::reset(void)
 {
   cl_uc::reset();
 
-  regs.SP = 0x7ff;
+  regs.SP = 0x17ff;
   regs.A = 0;
   regs.X = 0;
   regs.Y = 0;
   regs.CC = 0x00;
   regs.VECTOR = 1;
-
 }
 
 
@@ -180,7 +179,7 @@ cl_stm8::inst_length(t_addr addr)
 {
   int len = 0;
 
-  get_disasm_info(addr, &len, NULL, NULL);
+  get_disasm_info(addr, &len, NULL, NULL, NULL);
 
   return len;
 }
@@ -190,11 +189,21 @@ cl_stm8::inst_branch(t_addr addr)
 {
   int b;
 
-  get_disasm_info(addr, NULL, &b, NULL);
+  get_disasm_info(addr, NULL, &b, NULL, NULL);
 
   return b;
 }
 
+bool
+cl_stm8::is_call(t_addr addr)
+{
+  struct dis_entry *e;
+
+  get_disasm_info(addr, NULL, NULL, NULL, &e);
+
+  return e?(e->is_call):false;
+}
+
 int
 cl_stm8::longest_inst(void)
 {
@@ -204,9 +213,10 @@ cl_stm8::longest_inst(void)
 
 const char *
 cl_stm8::get_disasm_info(t_addr addr,
-                        int *ret_len,
-                        int *ret_branch,
-                        int *immed_offset)
+			 int *ret_len,
+			 int *ret_branch,
+			 int *immed_offset,
+			 struct dis_entry **dentry)
 {
   const char *b = NULL;
   uint code;
@@ -298,6 +308,9 @@ cl_stm8::get_disasm_info(t_addr addr,
   if (ret_len)
     *ret_len = len;
 
+  if (dentry)
+    *dentry= dis_e;
+  
   return b;
 }
 
@@ -312,7 +325,7 @@ cl_stm8::disass(t_addr addr, const char *sep)
 
   p= work;
 
-  b = get_disasm_info(addr, &len, NULL, &immed_offset);
+  b = get_disasm_info(addr, &len, NULL, &immed_offset, NULL);
 
   if (b == NULL) {
     buf= (char*)malloc(30);
@@ -574,14 +587,14 @@ cl_stm8::exec_inst(void)
                   regs.X |= (regs.A << 8);
                   regs.A = tempi & 0xff;
                   FLAG_ASSIGN (BIT_N, 0x8000 & regs.X);
-                  FLAG_ASSIGN (BIT_Z, regs.X ^ 0xffff);
+                  FLAG_ASSIGN (BIT_Z, regs.X == 0x0000);
                } else if (cprefix == 0x90) { // rrwa Y,A
                   tempi = regs.Y;
                   regs.Y >>= 8;
                   regs.Y |= (regs.A << 8);
                   regs.A = tempi & 0xff;
                   FLAG_ASSIGN (BIT_N, 0x8000 & regs.Y);
-                  FLAG_ASSIGN (BIT_Z, regs.Y ^ 0xffff);
+                  FLAG_ASSIGN (BIT_Z, regs.Y == 0x0000);
                } else {
                   return(resHALT);
                }
@@ -643,14 +656,14 @@ cl_stm8::exec_inst(void)
                   regs.X |= regs.A ;
                   regs.A = tempi >> 8;
                   FLAG_ASSIGN (BIT_N, 0x8000 & regs.X);
-                  FLAG_ASSIGN (BIT_Z, regs.X ^ 0xffff);
+                  FLAG_ASSIGN (BIT_Z, regs.X == 0x0000);
                } else if (cprefix == 0x90) { // rlwa Y,A
                   tempi = regs.Y;
                   regs.Y <<= 8;
                   regs.Y |= regs.A ;
                   regs.A = tempi >> 8;
                   FLAG_ASSIGN (BIT_N, 0x8000 & regs.Y);
-                  FLAG_ASSIGN (BIT_Z, regs.Y ^ 0xffff);
+                  FLAG_ASSIGN (BIT_Z, regs.Y == 0x0000);
                } else {
                   return(resHALT);
                }
@@ -700,11 +713,12 @@ cl_stm8::exec_inst(void)
             case 0x40:
             case 0x50:
             case 0x60:
-            case 0x70: // CPL
+            case 0x70: // CPL, CPLW
                return( inst_cpl( code, cprefix));
                break;
-            case 0x80: 
+            case 0x80: // TRAP
                // store to stack
+               PC++;
                push2( PC & 0xffff);
                push1( PC >> 16); //extended PC
                push2( regs.Y);
@@ -718,8 +732,8 @@ cl_stm8::exec_inst(void)
                PC = get1(0x8004);
                if (PC == 0x82) { // this is reserved opcode for vector table
                   regs.VECTOR = 0;
-                  PC = get1(0x8005)*(1<<16);
-                  PC += get2(0x8006);
+                  PC = get1(0x8005) << 16;
+                  PC |= get2(0x8006);
                   return(resGO);
                }
                return(resHALT);
@@ -754,7 +768,7 @@ cl_stm8::exec_inst(void)
             case 0x40:
             case 0x50:
             case 0x60:
-            case 0x70: // SRl
+            case 0x70: // SRL
                return( inst_srl( code, cprefix));
                break;
             case 0x80: 
@@ -804,10 +818,10 @@ cl_stm8::exec_inst(void)
                store1(opaddr, tempi);
                return( resGO);
                break;
-            case 0x60: // divw
+            case 0x60: // DIVW
                return( inst_div( code, cprefix));
                break;
-            case 0x80: 
+            case 0x80:
                if(cprefix==0x90) {
                   pop2(regs.Y);
                } else if(cprefix==0x00) {
@@ -851,7 +865,7 @@ cl_stm8::exec_inst(void)
             case 0x70: // RRC
                return( inst_rrc( code, cprefix));
                break;
-            case 0x10:       
+            case 0x10:
                return(inst_ldxy( code, cprefix));
                break;
             case 0x80: 
@@ -893,6 +907,8 @@ cl_stm8::exec_inst(void)
             case 0x10:
                opaddr = fetch1()+regs.SP;
                store2(opaddr, regs.Y);
+               FLAG_ASSIGN (BIT_Z, (regs.Y & 0xffff) == 0x0000);
+               FLAG_ASSIGN (BIT_N, regs.Y & 0x8000);
                return(resGO);
                break;
             case 0x80: // RETF
@@ -918,8 +934,10 @@ cl_stm8::exec_inst(void)
                   store1(get3(opaddr)+regs.Y,regs.A);
                } else if(cprefix==0x90) {
                   store1((opaddr << 8) + fetch() + regs.Y, regs.A);
-               } else {
+               } else if(cprefix==0x00) {
                   store1((opaddr << 8) + fetch() + regs.X, regs.A);
+               } else {
+                  return(resHALT);
                }
                FLAG_NZ (regs.A);
                return(resGO);
@@ -1001,7 +1019,6 @@ cl_stm8::exec_inst(void)
                printf("************* bad code !!!!\n");
                return(resINV_INST);
          }
-      
          break;
       case 0xa:
          switch ( code & 0xf0) {
@@ -1029,36 +1046,38 @@ cl_stm8::exec_inst(void)
             case 0xF0: // OR
                return( inst_or( code, cprefix));
                break;
-             default: 
+             default:
                printf("************* bad code !!!!\n");
                return(resINV_INST);
          }
          break;
       case 0xb:
          switch ( code & 0xf0) {
-            case 0x30: 
+            case 0x30: // push longmem
                push1( get1(fetch2()));
                return(resGO);
-            case 0x40: 
+            case 0x40: // push #byte
                push1( fetch1());
                return(resGO);
-            case 0x50: // add sp,#val
-               regs.SP += fetch();
+            case 0x50: // addw sp,#val
+               regs.SP += fetch1();
                return(resGO);
-               break;            
+               break;
             case 0x60: // ld (shortoff,SP),A
                store1(fetch1()+regs.SP, regs.A);
+               FLAG_NZ(regs.A);
                return(resGO);
-               break;            
+               break;
             case 0x70: // ld A,(shortoff,SP)
                regs.A = get1(fetch1()+regs.SP);
+               FLAG_NZ(regs.A);
                return(resGO);
-               break;            
+               break;
             case 0x90: // SIM - disable INT
                FLAG_SET(BIT_I0);
                FLAG_SET(BIT_I1);
                return(resGO);
-            case 0x10: 
+            case 0x10:
             case 0xA0:
             case 0xB0:
             case 0xC0:
@@ -1067,7 +1086,7 @@ cl_stm8::exec_inst(void)
             case 0xF0: // ADD
                return( inst_add( code, cprefix));
                break;
-            default: 
+            default:
                printf("************* bad code !!!!\n");
                return(resINV_INST);
          }
@@ -1137,12 +1156,15 @@ cl_stm8::exec_inst(void)
                break;
             case 0x80: // CALLF
                opaddr = fetch2();
-               push2(PC & 0xffff);
-               push1(PC>>16);
                if (cprefix == 0x92) {
-                  PC = get3(opaddr);
+                   push2(PC & 0xffff);
+                   push1(PC >> 16);
+                   PC = get3(opaddr);
                } else {
-                  PC = (opaddr << 8) + fetch();
+                   unsigned char c = fetch();
+                   push2(PC & 0xffff);
+                   push1(PC >> 16);
+                   PC = (opaddr << 8) + c;
                }
                return(resGO);
                break;
@@ -1150,9 +1172,12 @@ cl_stm8::exec_inst(void)
                return(resGO);
                break;
             case 0xA0: // CALLR
+             {
+               char c = (char) fetch1();
                push2(PC);
-               PC += (char) fetch1();
+               PC += c;
                return(resGO);
+             }
                break;            
             case 0xb0: // LDF
                opaddr = fetch2();
@@ -1188,11 +1213,11 @@ cl_stm8::exec_inst(void)
             case 0x80: 
                printf("************* HALT instruction reached !!!!\n");
                return(resHALT);
-            case 0x90:
+            case 0x90: // LD A, YH / XH
                if(cprefix==0x90) {
-                  regs.A = (regs.Y >> 8);
+                  regs.A = (regs.Y >> 8) & 0xff;
                } else if(cprefix==0x00) {
-                  regs.A = (regs.X >> 8);
+                  regs.A = (regs.X >> 8) & 0xff;
                } else {
                   return(resHALT);
                }
@@ -1207,7 +1232,7 @@ cl_stm8::exec_inst(void)
             case 0xF0: // LDXY
                return( inst_ldxy( code, cprefix));
                break;
-            default: 
+            default:
                printf("************* bad code !!!!\n");
                return(resINV_INST);
          }
@@ -1217,6 +1242,7 @@ cl_stm8::exec_inst(void)
             case 0x10:
                // ldw   (offset,SP),X
                return( inst_ldxydst( code, cprefix ) );
+               break;
             case 0x00: 
             case 0x30:
             case 0x40:
@@ -1245,8 +1271,10 @@ cl_stm8::exec_inst(void)
                   regs.A = get1(get3(opaddr)+regs.Y);
                } else if(cprefix==0x90) {
                   regs.A = get1((opaddr << 8) + fetch() + regs.Y);
-               } else {
+               } else if(cprefix==0x00) {
                   regs.A = get1((opaddr << 8) + fetch() + regs.X);
+               } else {
+                  return(resHALT);
                }
                FLAG_NZ (regs.A);
                return(resGO);
