@@ -145,7 +145,7 @@ enum {
   W_SHIFT_CHANGED               = 116, /* shift changed to zero */
   W_UNKNOWN_OPTION              = 117, /* don't know the option */
   W_UNSUPP_OPTION               = 118, /* processor reset has been redifned */
-  W_UNKNOWN_FEXT                = 119, /* unknown file extension */
+  E_UNKNOWN_FEXT                = 119, /* unknown file extension */
   W_TOO_MANY_SRC                = 120, /* can only compile one .c file at a time */
   I_CYCLOMATIC                  = 121, /* information message */
   E_DIVIDE_BY_ZERO              = 122, /* / 0 */
@@ -230,8 +230,8 @@ enum {
   W_DEPRECATED_OPTION           = 201, /* deprecated compiler option '%s' */
   E_BAD_DESIGNATOR              = 202, /* Bad designated initializer */
   W_DUPLICATE_INIT              = 203, /* duplicate initializer */
-  E_INVALID_UNIVERSAL           = 204, /* incomplete universal character name %s. */
-  W_UNIVERSAL_C99               = 205, /* universal character names are only valid in C99 */
+  E_INVALID_UNIVERSAL           = 204, /* invalid universal character name %s. */
+  W_UNIVERSAL_C95               = 205, /* universal character names are only valid in C95 or later */
   E_SHORTLONG                   = 206, /* Invalid combination of short / long */
   E_INTEGERSUFFIX               = 207, /* Invalid integer suffix */
   E_AUTO_ADDRSPACE              = 208, /* named address space for auto var */
@@ -245,6 +245,26 @@ enum {
   E_ALIGNAS                     = 216, /* invalid alignment specified */
   W_INTERNAL_ERROR              = 217, /* warning for non-fatal internal errors - things that should not have happened, but can be handled */
   W_UNRECOGNIZED_ASM            = 218, /* unrecognized asm instruction in peephole optimizer */
+  W_FLEXARRAY_INSTRUCT          = 219, /* using flexible arrays in a struct */
+  E_TYPE_IS_FUNCTION            = 220, /* typedef void foo_t(void); foo_t foo; */
+  W_INLINE_NAKED                = 221, /* inline function is naked */
+  E_Z88DK_FASTCALL_PARAMETERS   = 222, /* invalid number of parameters in __z88dk_fastcall */
+  E_Z88DK_FASTCALL_PARAMETER    = 223, /* invalid parameter type in __z88dk_fastcall */
+  W_REPEAT_QUALIFIER            = 224, /* the same qualifier appears more than once */
+  W_NO_TYPE_SPECIFIER           = 225, /* type specifier missing in declaration */
+  E_NO_TYPE_SPECIFIER           = 226, /* type specifier missing in declaration */
+  E_MULTIPLE_DEFAULT_IN_GENERIC = 227, /* multiple default expressions in generic association */
+  E_MULTIPLE_MATCHES_IN_GENERIC = 228, /* multiple matching expressions in generic association */
+  E_NO_MATCH_IN_GENERIC         = 229, /* no matching expression in generic association */
+  W_LABEL_WITHOUT_STATEMENT     = 230, /* label without statement, not allowed in standard C */
+  E_WCHAR_CONST_C95             = 231, /* character constant of type wchar_t requires ISO C 95 or later */
+  E_WCHAR_CONST_C11             = 232, /* character constant of type char16_t or char32_t equires ISO C 11 or later */
+  E_WCHAR_STRING_C95            = 233, /* wide character string literal requires ISO C 95 or later */
+  E_WCHAR_STRING_C11            = 234, /* wide character string literal requires ISO C 11 or later */
+  W_UNKNOWN_REG                 = 235, /* unknown register specified */
+  E_HEXFLOAT_C99                = 236, /* hexadecimal floating constant requires ISO C99 or later */
+  E_ANONYMOUS_STRUCT_TAG        = 237, /* anonymous struct/union should not have a tag */
+  W_INLINE_FUNCATTR             = 238, /* inline functions should not be z88dk_fastcall or z88dk_callee */
 
   /* don't touch this! */
   NUMBER_OF_ERROR_MESSAGES             /* Number of error messages */
