@@ -203,6 +203,7 @@ FBYNAME (labelInRange)
          -127 to + 127 bytes, for Z80 -126 to +129 bytes.*/
       dist = (pcDistance (currPl, lbl, TRUE) +
               pcDistance (currPl, lbl, FALSE));
+
       /* Use 125 for now. Could be made more exact using port and
          exact jump location instead of currPl. */
       if (!dist || dist > 127)
@@ -226,9 +227,6 @@ FBYNAME (labelJTInRange)
   char *lbl;
   int dist, count, i;
 
-  if (TARGET_IS_MCS51 && !getenv("SDCC_SJMP_JUMPTABLE"))
-    return FALSE;
-
   /* Only optimize within a jump table */
   if (currPl->ic && currPl->ic->op != JUMPTABLE)
     return FALSE;
@@ -315,7 +313,7 @@ FBYNAME (labelIsReturnOnly)
   if (!pl)
     return FALSE; /* did not find the label */
   pl = pl->next;
-  while (pl && (pl->isDebug || pl->isComment))
+  while (pl && (pl->isDebug || pl->isComment || pl->isLabel))
     pl = pl->next;
   if (!pl || !pl->line || pl->isDebug)
     return FALSE; /* next line not valid */
@@ -341,6 +339,7 @@ FBYNAME (labelIsUncondJump)
   const char *label;
   char *p, *q;
   const lineNode *pl;
+  bool found = FALSE;
   int len;
   char * jpInst = NULL;
   char * jpInst2 = NULL;
@@ -350,23 +349,48 @@ FBYNAME (labelIsUncondJump)
     return FALSE;
   len = strlen(label);
 
-  for (pl = currPl; pl; pl = pl->next)
+  for (pl = currPl; pl; pl = pl->prev)
     {
       if (pl->line && !pl->isDebug && !pl->isComment && pl->isLabel)
         {
           if (strncmp(pl->line, label, len) == 0)
-            break; /* Found Label */
+            {
+              found = TRUE;
+              break; /* Found Label */
+            }
           if (strlen(pl->line) != 7       || !ISCHARDIGIT(*(pl->line))   ||
               !ISCHARDIGIT(*(pl->line+1)) || !ISCHARDIGIT(*(pl->line+2)) ||
               !ISCHARDIGIT(*(pl->line+3)) || !ISCHARDIGIT(*(pl->line+4)) ||
               *(pl->line+5) != '$')
             {
-              return FALSE; /* non-local label encountered */
+              break; /* non-local label encountered */
             }
         }
     }
 
-  if (!pl)
+  if (!found)
+    {
+      for (pl = currPl; pl; pl = pl->next)
+        {
+          if (pl->line && !pl->isDebug && !pl->isComment && pl->isLabel)
+            {
+              if (strncmp(pl->line, label, len) == 0)
+                {
+                  found = TRUE;
+                  break; /* Found Label */
+                }
+              if (strlen(pl->line) != 7       || !ISCHARDIGIT(*(pl->line))   ||
+                  !ISCHARDIGIT(*(pl->line+1)) || !ISCHARDIGIT(*(pl->line+2)) ||
+                  !ISCHARDIGIT(*(pl->line+3)) || !ISCHARDIGIT(*(pl->line+4)) ||
+                  *(pl->line+5) != '$')
+                {
+                  return FALSE; /* non-local label encountered */
+                }
+            }
+        }
+    }
+
+  if (!pl || !found)
     return FALSE; /* did not find the label */
   pl = pl->next;
   while (pl && (pl->isDebug || pl->isComment))
@@ -378,7 +402,10 @@ FBYNAME (labelIsUncondJump)
     p++;
 
   if (TARGET_MCS51_LIKE)
-    jpInst = "ljmp";
+    {
+      jpInst = "ljmp";
+      jpInst2 = "sjmp";
+    }
   else if (TARGET_HC08_LIKE)
     {
       jpInst = "jmp";
@@ -958,6 +985,8 @@ operandBaseName (const char *op)
         return "iy";
       if (!strcmp (op, "ixh") || !strcmp (op, "ixl") || strstr (op, "ix"))
         return "ix";
+      if (!strcmp (op, "a"))
+        return "af";
     }
 
   return op;
@@ -1107,6 +1136,53 @@ FBYNAME (operandsNotRelated)
   return TRUE;
 }
 
+/*-----------------------------------------------------------------*/
+/* notSimilar - Check, if one is another's substring               */
+/*-----------------------------------------------------------------*/
+FBYNAME (notSimilar)
+{
+  set *operands;
+  const char *op1, *op2;
+
+  operands = setFromConditionArgs (cmdLine, vars);
+
+  if (!operands)
+    {
+      fprintf (stderr,
+               "*** internal error: notSame peephole restriction"
+               " malformed: %s\n", cmdLine);
+      return FALSE;
+    }
+
+  while ((op1 = setFirstItem (operands)))
+    {
+      deleteSetItem (&operands, (void*)op1);
+
+      for (op2 = setFirstItem (operands); op2; op2 = setNextItem (operands))
+        {
+          if ((strstr (op1, op2) || strstr (op2, op1)) && strcmp (op1, op2) == 0)
+            {
+              deleteSet (&operands);
+              return FALSE;
+            }
+        }
+    }
+
+  deleteSet (&operands);
+  return TRUE;
+}
+
+/*-----------------------------------------------------------------*/
+/* symmParmStack - Caller readjusts stack by the number of bytes
+   that were pushed in all calls to this function                  */
+/*-----------------------------------------------------------------*/
+FBYNAME (symmParmStack)
+{
+  if (port->peep.symmParmStack)
+    return port->peep.symmParmStack();
+  return FALSE;
+}
+
 /*-----------------------------------------------------------------*/
 /* notSame - Check, that arguments are pairwise not the same       */
 /*-----------------------------------------------------------------*/
@@ -1175,6 +1251,161 @@ FBYNAME (operandsLiteral)
   return TRUE;
 }
 
+static long *
+immdGet (const char *pc, long *pl)
+{
+  long s = 1;
+
+  if (!pc || !pl)
+    return NULL;
+
+  // omit space
+  for (; ISCHARSPACE (*pc); pc++);
+  // parse sign
+  for (; !ISCHARDIGIT (*pc); pc++)
+    if (*pc == '-')
+      s *= -1;
+  else if (*pc == '+')
+      s *= +1;
+  else
+    return NULL;
+
+  if (pc[0] == '0' && (pc[1] == 'x' || pc[1] == 'X'))
+    {
+      if (sscanf (pc + 2, "%lx", pl) != 1)
+        return NULL;
+    }
+  else
+    {
+      if (sscanf (pc, "%ld", pl) != 1)
+        return NULL;
+    }
+
+  *pl *= s;
+  return pl;
+}
+
+static bool
+immdError (const char *info, const char *param, const char *cmd)
+{
+  fprintf (stderr, "*** internal error: immdInRange gets "
+           "%s: \"%s\" in \"%s\"\n", info, param, cmd);
+  return FALSE;
+}
+
+/*-----------------------------------------------------------------*/
+/* immdInRange - returns true if the sum or difference of two      */
+/* immediates is in a give range.                                  */
+/*-----------------------------------------------------------------*/
+FBYNAME (immdInRange)
+{
+  char r[64], operator[8];
+  const char *op;
+  long i, j, k, h, low, high, left_l, right_l, order;
+  const char *padd[] = {"+", "'+'", "\"+\"", "add", "'add'", "\"add\""};
+  const char *psub[] = {"-", "'-'", "\"-\"", "sub", "'sub'", "\"sub\""};
+
+  for (i = order = 0; order < 6;)
+    {
+      // pick up one parameter in the temp buffer r[64]
+      for (; ISCHARSPACE (cmdLine[i]) && cmdLine[i]; i++);
+      for (j = i; !ISCHARSPACE (cmdLine[j]) && cmdLine[j]; j++);
+      if (!cmdLine[i]) // unexpected end
+        return immdError ("no enough input", "", cmdLine);
+      else
+        {
+          for (k = i; k < j; k++)
+            r[k - i] = cmdLine[k];
+          r[j - i] = 0;
+        }
+      // parse the string by order
+      switch (order)
+        {
+          case 0: // lower bound
+            if (!immdGet (r, &low))
+              return immdError ("bad lower bound", r, cmdLine);
+            break;
+          case 1: // upper bound
+            if (!immdGet (r, &high))
+              return immdError ("bad upper bound", r, cmdLine);
+            break;
+          case 2: // operator
+            if (sscanf (r, "%s", operator) != 1)
+              return immdError ("bad operator", r, cmdLine);
+            break;
+          case 3: // left operand
+            if (immdGet (r, &left_l)) // the left operand is given directly
+              {
+              }
+            else if (r[0] == '%') // the left operand is passed via pattern match
+              {
+                if (!immdGet (r + 1, &k) || !(op = hTabItemWithKey (vars, (int) k)))
+                  return immdError ("bad left operand", r, cmdLine);
+                else if (!immdGet (op, &left_l))
+                  return immdError ("bad left operand", op, r);
+              }
+            else
+              return immdError ("bad left operand", r, cmdLine);
+            break;
+          case 4: // right operand
+            if (immdGet (r, &right_l)) // the right operand is given directly
+              {
+              }
+            else if (r[0] == '%') // the right operand is passed via pattern match
+              {
+                if (!immdGet (r + 1, &k) || !(op = hTabItemWithKey (vars, (int) k)))
+                  return immdError ("bad right operand", r, cmdLine);
+                else if (!immdGet (op, &right_l))
+                  return immdError ("bad right operand", op, r);
+              }
+            else
+              return immdError ("bad right operand", r, cmdLine);
+            break;
+          case 5: // result
+            if (r[0] != '%' || !immdGet (r + 1, &h))
+              return immdError ("bad result container", r, cmdLine);
+            break;
+          default: // should not reach
+            return immdError ("unexpected input", "", cmdLine);
+            break;
+        }
+      order++;
+      i = j;
+    }
+
+  // calculate
+  for (j = k = 0; k < sizeof (padd) / sizeof (padd[0]); k++) // add
+    if (strcmp (operator, padd[k]) == 0)
+      {
+        i = left_l + right_l;
+        j = 1;
+        break;
+      }
+  if (!j)
+    for (k = 0; k < sizeof (psub) / sizeof (psub[0]); k++) // sub
+      if (strcmp (operator, psub[k]) == 0)
+        {
+          i = left_l - right_l;
+          j = 1;
+          break;
+        }
+  if (!j)
+    return immdError ("bad operator", operator, cmdLine);
+
+  // bind the result
+  if ((low <= i && i <= high) || (high <= i && i <= low))
+    {
+      char *p[] = {r, NULL};
+      sprintf (r, "%ld", i);
+      bindVar ((int) h, p, &vars);
+      return TRUE;
+    }
+  else
+    {
+      return FALSE;
+    }
+}
+
 static const struct ftab
 {
   char *fname;
@@ -1241,7 +1472,16 @@ ftab[] =                                            // sorted on the number of t
   },
   {
     "okToRemoveSLOC", okToRemoveSLOC                // 0
-  }
+  },
+  {
+    "immdInRange", immdInRange
+  },
+  {
+    "notSimilar", notSimilar
+  },
+  {
+    "symmParmStack", symmParmStack
+  },
 };
 
 /*-----------------------------------------------------------------*/
@@ -1538,8 +1778,13 @@ top:
   if (strncmp (bp, "if", 2) == 0)
     {
       bp += 2;
-      while ((ISCHARSPACE (*bp) || *bp == '\n') && *bp)
+      while ((ISCHARSPACE (*bp) || *bp == '\n' || (*bp == '/' && *(bp+1) == '/')) && *bp)
+      {
         bp++;
+        if (*bp == '/')
+          while (*bp && *bp != '\n')
+            bp++;
+      }
       if (!*bp)
         {
           fprintf (stderr, "expected condition name\n");
