@@ -29,8 +29,10 @@
 #include "dbuf.h"
 
 #define INTNO_MAX 255           /* maximum allowed interrupt number */
+#define INTNO_TRAP INTNO_MAX
 #define INTNO_UNSPEC (INTNO_MAX+1)      /* interrupt number unspecified */
 
+
 #define BITVAR_PAD -1
 
 enum
@@ -183,7 +185,9 @@ typedef struct specifier
   union
   {                                   /* Values if constant or enum */
     TYPE_TARGET_INT v_int;            /* 2 bytes: int and char values            */
-    const char *v_char;               /*          character string               */
+    const char *v_char;               /*          char character string          */
+    const TYPE_TARGET_UINT *v_char16; /*          char16_t character string      */
+    const TYPE_TARGET_ULONG *v_char32;/*          char32_t character string      */
     TYPE_TARGET_UINT v_uint;          /* 2 bytes: unsigned int const value       */
     TYPE_TARGET_LONG v_long;          /* 4 bytes: long constant value            */
     TYPE_TARGET_ULONG v_ulong;        /* 4 bytes: unsigned long constant value   */
@@ -267,12 +271,15 @@ typedef struct sym_link
     unsigned inlinereq:1;           /* inlining requested                   */
     unsigned noreturn:1;            /* promised not to return               */
     unsigned smallc:1;              /* Parameters on stack are passed in reverse order */
+    unsigned z88dk_fastcall:1;      /* For the z80-related ports: Function has a single paramter of at most 32 bits that is passed in dehl */
+    unsigned z88dk_callee:1;        /* Stack pointer adjustment for parameters passed on the stack is done by the callee */
     unsigned intno;                 /* 1=Interrupt service routine          */
     short regbank;                  /* register bank 2b used                */
     unsigned builtin;               /* is a builtin function                */
     unsigned javaNative;            /* is a JavaNative Function (TININative ONLY) */
     unsigned overlay;               /* force parameters & locals into overlay segment */
     unsigned hasStackParms;         /* function has parameters on stack     */
+    bool preserved_regs[9];         /* Registers preserved by the function - may be an underestimate */
   } funcAttrs;
 
   struct sym_link *next;            /* next element on the chain  */
@@ -330,6 +337,7 @@ typedef struct symbol
   unsigned spildir:1;               /* spilt in direct space */
   unsigned ptrreg:1;                /* this symbol assigned to a ptr reg */
   unsigned noSpilLoc:1;             /* cannot be assigned a spil location */
+  unsigned div_flag_safe:1;         /* we know this function is safe to call with undocumented stm8 flag bit 6 set*/
   unsigned isstrlit;                /* is a string literal and it's usage count  */
   unsigned accuse;                  /* can be left in the accumulator
                                        On the Z80 accuse is divided into
@@ -442,6 +450,10 @@ extern sym_link *validateLink (sym_link * l,
 #define IFFUNC_ISOVERLAY(x) (IS_FUNC(x) && FUNC_ISOVERLAY(x))
 #define FUNC_ISSMALLC(x) (x->funcAttrs.smallc)
 #define IFFUNC_ISSMALLC(x) (IS_FUNC(x) && FUNC_ISSMALLC(x))
+#define FUNC_ISZ88DK_FASTCALL(x) (x->funcAttrs.z88dk_fastcall)
+#define IFFUNC_ISZ88DK_FASTCALL(x) (IS_FUNC(x) && FUNC_ISZ88DK_FASTCALL(x))
+#define FUNC_ISZ88DK_CALLEE(x) (x->funcAttrs.z88dk_callee)
+#define IFFUNC_ISZ88DK_CALLEE(x) (IS_FUNC(x) && FUNC_ISZ88DK_CALLEE(x))
 
 #define BANKED_FUNCTIONS        ( options.model == MODEL_HUGE || \
                                   ( (options.model == MODEL_LARGE || options.model == MODEL_MEDIUM) && \
@@ -454,6 +466,7 @@ extern sym_link *validateLink (sym_link * l,
 #define SPEC_LONGLONG(x) validateLink(x, "SPEC_LONGLONG", #x, SPECIFIER, __FILE__, __LINE__)->select.s.b_longlong
 #define SPEC_SHORT(x) validateLink(x, "SPEC_LONG", #x, SPECIFIER, __FILE__, __LINE__)->select.s.b_short
 #define SPEC_USIGN(x) validateLink(x, "SPEC_USIGN", #x, SPECIFIER, __FILE__, __LINE__)->select.s.b_unsigned
+#define SPEC_SIGN(x) validateLink(x, "SPEC_USIGN", #x, SPECIFIER, __FILE__, __LINE__)->select.s.b_signed
 #define SPEC_SCLS(x) validateLink(x, "SPEC_SCLS", #x, SPECIFIER, __FILE__, __LINE__)->select.s.sclass
 #define SPEC_ENUM(x) validateLink(x, "SPEC_ENUM", #x, SPECIFIER, __FILE__, __LINE__)->select.s.b_isenum
 #define SPEC_OCLS(x) validateLink(x, "SPEC_OCLS", #x, SPECIFIER, __FILE__, __LINE__)->select.s.oclass
@@ -690,7 +703,7 @@ void addSym (bucket **, void *, char *, int, int, int checkType);
 void deleteSym (bucket **, void *, const char *);
 void *findSym (bucket **, void *, const char *);
 void *findSymWithLevel (bucket **, struct symbol *);
-void *findSymWithBlock (bucket **, struct symbol *, int);
+void *findSymWithBlock (bucket **, struct symbol *, int, int);
 void changePointer (sym_link * p);
 void checkTypeSanity (sym_link * etype, const char *name);
 sym_link *typeFromStr (const char *);
